% create graph for shortest path through video

%clear all; pack;
minWnSz = 128;   % min size of square cropping window [was=32 or 64]
maxWnSz = 128;  % max size of square cropping window [was=200 or 240]
                % (with res=16, this makes 11 steps)
% works on frame differences
%pathIn  = 'C:\Documents and Settings\helalfy\My Documents\Research\Thumbnailing Frames\FrameDiffClosed';
pathIn  = 'C:\MunichFrmDif\Scene2Cam1';
camera  = 'Cam1-'; frmt = 'png';    % camera number and file format
fftest  = imread([pathIn '\' camera sprintf('%05d',1) '.' frmt]);   % read test frame
ImSz    = size(fftest);     % frame size
res     = 16;        % resolution: pixels moved by cropping rectangle per step (was =8)
belt    = res;    % belt width around the window to compute Laplacian
rt1     = 1;
rt2     = 1;        % aspect ratio of rectangle in height dimension (i/j)
nghb    = 11;       % number of permissible neighbors (self + 8 neighbors + 2 size variations)
nFrames = 2000;      % number of frames allowed per graph [Sc1:0-552, Sc2:560-992]
stp     = 4;        % step between frames to be computed (was =2 for AVW)
dT      = floor(nFrames/stp);   % number of frames actually processed
offset  = 0/stp;	% frame after which to start processing; normalized by 'stp'
nSz     = floor((maxWnSz-minWnSz)/(2*res))+1;     % no of different window sizes
VinVss  = floor(nSz/2)+2;   % size of video-in-video

% create system graph (adjacency list)
fprintf('Building graph...\n'); tic;
% initialize data structure
GR      = cell(nSz,1);  % graph mapping the video; each window size in a layer
WnSz    = minWnSz;      % variable window size
for ssw = 1:nSz
    margin  = WnSz/2;   % frame margin where no windows allowed
    nH      = floor((ImSz(1)-2*margin)/res)+1;    % number of rows of windows
    nW      = floor((ImSz(2)-2*margin)/res)+1;    % number of cols of windows
    nWin    = nH*nW;	% no of windows in a frame using windows of size WnSz
    GR{ssw} = struct('next',zeros(nWin,2*nghb,dT), 'cost',zeros(nWin,nghb,dT));
    WnSz    = WnSz + (2*res);   % difference between consecutive window sizes
    if ssw==VinVss, VinV=zeros(1,nWin); end     % best location for video-in-video
end
% in Help, it says that the above saves significant memory over this:
% GR      = repmat(struct('next',0, 'cost',0), [nWin,nghb,dT]);
% create edges and assign weights
for tt  = 1:dT-1      % for each frame in video
    fprintf('Processing frame %5d of %5d', tt,dT)
%     thisfr  = imread([pathIn '\' camera sprintf('%05d',stp*(tt+offset)-(stp-1)) '.' frmt]);  % current diff image
    nextfr  = imread([pathIn '\' camera sprintf('%05d',stp*(tt+offset)+1) '.' frmt]); % next diff image
    % integral images of frames
%     thisfr  = cumsum(cumsum(thisfr),2);
    nextfr  = cumsum(cumsum(nextfr),2);
    for ssw=1:nSz               % for each possible window size, and ...
        WnSz    = minWnSz + (ssw-1)*(2*res);   % "original" square window sizes
        marg1   = WnSz/2;       % frame margin where no windows allowed
        WnArea  = 2*marg1 * (rt1+rt2)*marg1;        % area of *rectangle* window
        nH      = floor((ImSz(1)-2*marg1)/res)+1;     % number of rows of windows
        nW      = floor((ImSz(2)-2*marg1)/res)+1;     % number of cols of windows
        nWin    = nH*nW;        % no of windows in a frame using windows of size WnSz
        % accumulate energy for best video-in-video location
        if ssw == VinVss
            for ww  = 1:nWin
                % coordinates of center of window 'ww'
                jj  = mod((ww-1),nW)+1;     ii  = floor((ww-1)/nW)+1;
                jj  = (jj-1)*res+marg1;     ii  = (ii-1)*res+marg1;
                % window boundaries and energy
                Li  = jj - margin;      Ri  = jj + margin;
                Ti  = ii - rt1*margin;  Bi  = ii + rt2*margin;
                Li  = max(Li,1);    Ri  = min(Ri,ImSz(2));  % limit to frame
                Ti  = max(Ti,1);    Bi  = min(Bi,ImSz(1));  %    boundary
                Ein = nextfr(Bi,Ri)+nextfr(Ti,Li)-nextfr(Ti,Ri)-nextfr(Bi,Li);
                VinV(ww) = VinV(ww) + Ein;  % accumulate energy for this frame
            end
        end
        for ww  = 1:nWin        % for each window location in this frame
            % get neighbors' list for input window in (i,j)-coordinates and
            % window number formats [variable size list 4, 6, 9 or 11]
            [nblst] = neighb(ww,ssw, nH,nW, nSz);
            for nn  = 1:size(nblst,1)   % for all possible neighbors of window 'ww'
                margin    = marg1 + (nblst(nn,4)-ssw)*res;  % neighbor's margin
                WnArea_nb = 2*margin * (rt1+rt2)*margin;    % neighbor's area
                % center pixel
                ii = (nblst(nn,2)-1)*res+margin;   jj = (nblst(nn,3)-1)*res+margin;
                % energy in next window computed as in Viola & Jones
                % outside window boundaries and energy
                Lo  = jj - margin-belt;       Ro  = jj + margin+belt;
                To  = ii - rt1*margin-belt; Bo  = ii + rt2*margin+belt;
                Lo  = max(Lo,1);    Ro  = min(Ro,ImSz(2));  % limit to frame
                To  = max(To,1);    Bo  = min(Bo,ImSz(1));  %    boundary
                Eout= nextfr(Bo,Ro)+nextfr(To,Lo)-nextfr(To,Ro)-nextfr(Bo,Lo);
                % inside window boundaries and energy
                Li  = jj - margin;      Ri  = jj + margin;
                Ti  = ii - rt1*margin;  Bi  = ii + rt2*margin;
                Li  = max(Li,1);    Ri  = min(Ri,ImSz(2));  % limit to frame
                Ti  = max(Ti,1);    Bi  = min(Bi,ImSz(1));  %    boundary
                Ein = nextfr(Bi,Ri)+nextfr(Ti,Li)-nextfr(Ti,Ri)-nextfr(Bi,Li);
                % cost function: Laplacian of empty percentage
                KK      = WnArea_nb/4;
                cost    = round(100*(1 - max((1+KK)*Ein-KK*Eout,0)/WnArea_nb));
                % add an edge to the graph; if (GR.next==0) edge doesn't exist
                GR{ssw}.next(ww,2*nn-1,tt)=nblst(nn,1); % neighbor window
                GR{ssw}.next(ww,2*nn,tt)=nblst(nn,4);   % neighbor layer (size)
                GR{ssw}.cost(ww,nn,tt)=cost;            % edge weight
            end     % for-nn
        end     % for-ww
    end     % for-ssw
    fprintf('\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b')
end     % for-tt
% last frame vertices connected to dummy sentinel vertex
for ssw=1:nSz               % for each possible window size
    WnSz    = minWnSz + (ssw-1)*(2*res);        % window size
    marg1   = WnSz/2;       % frame margin where no windows allowed
    nH      = floor((ImSz(1)-2*marg1)/res)+1;     % number of rows of windows
    nW      = floor((ImSz(2)-2*marg1)/res)+1;     % number of cols of windows
    nWin    = nH*nW;        % no of windows in a frame using windows of size WnSz
    GR{ssw}.next(:,:,dT) = Inf*ones(nWin,2*nghb);
end
toc;

% shortest path through graph
fprintf('Computing shortest path...\n');    tic
% vertex list with fields: distance to source node 'dist',
% predecessor node in shortest path 'pred', and last-updated 'updt'
for ssw = 1:nSz
    WnSz    = minWnSz + (ssw-1)*(2*res);        % window size
    margin  = WnSz/2;       % frame margin where no windows allowed
    nH      = floor((ImSz(1)-2*margin)/res)+1;    % number of rows of windows
    nW      = floor((ImSz(2)-2*margin)/res)+1;    % number of cols of windows
    nWin    = nH*nW;        % no of windows in a frame using windows of size WnSz
    V{ssw}  = struct('dist',Inf*ones(nWin,dT), 'pred',zeros(2*nWin,dT), 'updt',zeros(nWin,dT));
end
% stack to visit nodes in a depth first approach
maxC     = 101;     % different values of 'cost' from 0 to 100 inclusive
Buk      = cell(maxC,1);  % maxC buckets; cell vector of *empty* matrices
BukEntry = zeros(1,3);    % window number, size and frame number per vertex
% add dummy header node with cost=0; compute distances of first frame nodes
% first difference frame
firstfr  = imread([pathIn '\' camera sprintf('%05d',stp*(0+offset)+1) '.' frmt]);
firstfr  = cumsum(cumsum(firstfr),2);   % integral image
% first layer of nodes connected to dummy node
for ssw=1:nSz               % for each possible window size
    WnSz    = minWnSz + (ssw-1)*(2*res);        % window sizes
    margin  = WnSz/2;       % frame margin where no windows allowed
    WnArea  = 2*margin * (rt1+rt2)*margin;      % cropping window area in pixels
    nH      = floor((ImSz(1)-2*margin)/res)+1;    % number of rows of windows
    nW      = floor((ImSz(2)-2*margin)/res)+1;    % number of cols of windows
    nWin    = nH*nW;        % no of windows in a frame using windows of size WnSz
    for ww  = 1:nWin
        % compute center pixel of this window
        jj  = mod((ww-1),nW)+1;    % column of window center
        ii  = floor((ww-1)/nW)+1;  % row of window center
        ii  = (ii-1)*res+margin;    jj  = (jj-1)*res+margin;    % actual center's pixel
        % energy in current window computed as in Viola & Jones
        % outside window boundaries and energy
        Lo  = jj - margin-belt;     Ro  = jj + margin+belt;
        To  = ii - rt1*margin-belt; Bo  = ii + rt2*margin+belt;
        Lo  = max(Lo,1);            Ro  = min(Ro,ImSz(2));  % limit to frame
        To  = max(To,1);            Bo  = min(Bo,ImSz(1));  %    boundary
        Eout= firstfr(Bo,Ro)+firstfr(To,Lo)-firstfr(To,Ro)-firstfr(Bo,Lo);
        % inside window boundaries and energy
        Li  = jj - margin;      Ri  = jj + margin;
        Ti  = ii - rt1*margin;  Bi  = ii + rt2*margin;
        Li  = max(Li,1);        Ri  = min(Ri,ImSz(2));  % limit to frame
        Ti  = max(Ti,1);        Bi  = min(Bi,ImSz(1));  %    boundary
        Ein = firstfr(Bi,Ri)+firstfr(Ti,Li)-firstfr(Ti,Ri)-firstfr(Bi,Li);
        % cost function: Laplacian of empty percentage
        KK  = WnArea/4;
        cost= round(100*(1 - max((1+KK)*Ein-KK*Eout,0)/WnArea));
        BukEntry(1) = ww;       % window number
        BukEntry(2) = ssw;      % window size number       
        BukEntry(3) = 1;        % window frame number
        % hash into corresponding bucket according to 'cost'
        % indices in Matlab start at 1 not 0
        Buk{cost+1} = [Buk{cost+1}; BukEntry];    % append to bucket
        V{ssw}.dist(ww,1) = cost;           % all V.pred=0 unchanged
    end
end
% Dijkstra's main loop - with Dial's implementation (hashing)
maxtt   = 0;    % keep track of max depth of visited nodes
maxchg  = true; % has 'maxtt' been updated in this iteration? (for display)
%haponce = 0;    % flag for debug; no longer used
ptr     = 1;    % pointer to current minimum bucket
maxstk  = 0;    % max stack depth
tt      = 1;    % frame number
while 1     % terminate when last a node in last frame is reached
    if maxchg
        fprintf('Processing frame %5d of %5d', maxtt,dT)
    end
%    [minDist,minIdx]    = min(Stk(:,1));    % find next closest node to source
    % window coordinates (number, size and frame)
%    ww  = Stk(minIdx,2);   ssw = Stk(minIdx,3);   tt = Stk(minIdx,4);
%    Stk(minIdx,:) = [];         % entry will no longer be used
%    ww  = Stk(end,2);   ssw = Stk(end,3);   tt = Stk(end,4);
%    Stk(end,:) = [];
    % get next minimum bucket. Originally: ptr=mod(ptr-1+1,maxC)+1;
    while isempty(Buk{ptr}), ptr = mod(ptr,maxC)+1; end
    ww  = Buk{ptr}(1,1);   ssw = Buk{ptr}(1,2);   tt = Buk{ptr}(1,3);
    Buk{ptr}(1,:)   = [];   % entry is permanent and will no longer be used
    if tt==dT, break, end       % reached last frame
    for nn  = 1:nghb            % for all edges from this node
        wwnb=GR{ssw}.next(ww,2*nn-1,tt); % neighbor window number 
        sswnb=GR{ssw}.next(ww,2*nn,tt);  % neighbor window size
        ttnb=tt+1;                  % neighbor window time frame
        if wwnb==0, break, end      % no neighbor; boundary window
        % distance check
        if V{ssw}.dist(ww,tt) + GR{ssw}.cost(ww,nn,tt) < V{sswnb}.dist(wwnb,ttnb)
            % update neighbor's dist
            V{sswnb}.dist(wwnb,ttnb) = V{ssw}.dist(ww,tt) + GR{ssw}.cost(ww,nn,tt);
            % update neighbor's predecessor: 'ww' implicitly at frame 'tt'
            V{sswnb}.pred(2*wwnb-1,ttnb) = ww;  % pred window number
            V{sswnb}.pred(2*wwnb,ttnb)  = ssw;  % pred window size
            % hash into corresponding bucket according to 'dist'
            % indices in Matlab start at 1 not 0
            hash        = mod(V{sswnb}.dist(wwnb,ttnb),maxC)+1;
            Buk{hash}	= [Buk{hash}; wwnb,sswnb,ttnb];    % append to bucket
            % push on stack: distance, window number, size and frame of neighbor
            % no need to check if pushed node is already in 'Stk' because
            % all edges (i,j) have the same cost which depends on node 'j'
%             ff1=find(Stk(:,2)==wwnb); ff2=find(Stk(ff1,3)==sswnb);
%             ff3=find(Stk(ff1(ff2),4)==ttnb);
%             if length(ff3)>0
%                 haponce=haponce+1;
%             else
%                 Stk = [V{sswnb}.dist(wwnb,ttnb), wwnb,sswnb,ttnb ; Stk];
%             end
        end
    end     % for neighbors
    if ttnb>maxtt, maxtt=ttnb; maxchg=true; % max depth in graph
    else maxchg=false; end      % don't change display
    V{ssw}.updt(ww,tt)   = maxtt;    % if zero then node hasn't been visited
%     if size(Stk,1)>maxstk, maxstk=size(Stk,1); end
    if maxchg
        fprintf('\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b')
    end
end     % while graph nodes
fprintf('\n');

% Build path: we have final node's (ww,sww,tt)
path=zeros(2,dT);   % path with exactly 'dT' steps; win# and size# at each step
cost=zeros(1,dT);   % cumulative cost of path
path(1,dT) = ww;     path(2,dT) = ssw;       % last step initialized
for tt  = dT:-1:2   % build path backwards
    pp  = V{ssw}.pred(2*ww-1,tt);   % predecessor window number
    pps = V{ssw}.pred(2*ww,tt);     % predecessor window size
    if pp==0
        disp('Incomplete path. Broken link')
        return
    end
    path(1,tt-1) = pp;   path(2,tt-1) = pps;     % prepend to path
    cost(tt) = V{ssw}.dist(ww,tt);  % cumulative cost of path
    ww  = pp;   ssw = pps;          % iterate: prev window, prev size
end
cost(1) = V{ssw}.dist(ww,1);

% recover path, TRUE IMAGE COORDINATES, then smooth it
jjpath  = zeros(1,dT);  iipath = zeros(1,dT);  sspath = zeros(1,dT);
for tt  = 1:dT
    ssw = path(2,tt);     % window size
    % number of cols of windows at that scale
    WnSz    = minWnSz + (ssw-1)*(2*res);    % window sizes
    margin  = WnSz/2;       % frame margin where no windows allowed
    nW      = floor((ImSz(2)-WnSz)/res)+1;    % number of cols of windows
    % recover components, true image (pixel) coordinates
    jjpath(tt)=mod((path(1,tt)-1),nW)+1; jjpath(tt)=(jjpath(tt)-1)*res+margin;
    iipath(tt)=floor((path(1,tt)-1)/nW)+1; iipath(tt)=(iipath(tt)-1)*res+margin;
    sspath(tt)=WnSz;
end
% smooth each component separately. 'floor' smoothes better than 'round'
% moving average smoothing with neighborhood size=9 to smooth jiggles
% Using 'floor' with 'smooth' in addition to the 'round' with 'interp1'
% helps better remove the jiggles
jjsmooth=floor(smooth(jjpath,6*stp)); iismooth=floor(smooth(iipath,6*stp));
sssmooth=floor(smooth(sspath,6*stp));   % don't know if needed??
% sampling indices, includes first and last to avoid extrapolation
smpl=[1:5:dT]; if smpl(end)<dT, smpl=[smpl,dT];  end
% Hermite cubic interpolation of path over actual frames' pixels
jjpix = round(interp1( smpl,jjsmooth(smpl), [1:1/stp:dT], 'pchip' ));
iipix = round(interp1( smpl,iismooth(smpl), [1:1/stp:dT], 'pchip' ));
sspix = round(interp1( smpl,sssmooth(smpl), [1:1/stp:dT], 'pchip' ));
% last step actual frames (non interpolated)
lastjjpix=jjpix(end);   lastiipix=iipix(end);   lastsspix=sspix(end);
for ss=1:stp-1
    jjpix=[jjpix,lastjjpix]; iipix=[iipix,lastiipix]; sspix=[sspix,lastsspix];
end
 %pathsmooth=(iismooth-1)*nW+jjsmooth;    % recombine in window numbers
toc;    % time to compute shortest path

% Pass I: Wipe enclosed "objects" from source frame differences
fprintf('Wiping frames, Pass I...\n');    tic
%pathWipe='C:\Documents and Settings\helalfy\My Documents\Research\Thumbnailing Frames\wipedFrameDiff';
pathWipe='C:\MunichWiped';
if isempty(dir(pathWipe))
	mkdir(pathWipe);     % create output directory if not existent
end
for tt  = 1:dT
    % border of cropping window; avoid that they round off out of window
    margin = sspix(stp*tt-(stp-1))/2;
    L=jjpix(stp*tt-(stp-1))-margin;     R=jjpix(stp*tt-(stp-1))+margin;
    T=iipix(stp*tt-(stp-1))-margin;     B=iipix(stp*tt-(stp-1))+margin;
    L=max(L,1);     R=min(R,ImSz(2));   T=max(T,1);     B=min(B,ImSz(1));
    % read one image difference frame
    fftest = imread([pathIn '\' camera sprintf('%05d',stp*tt-(stp-1)) '.png']);
    [Lab,ncmpn] = bwlabel(fftest,8);  % connected components
    for cm  = 1:ncmpn
        [r,c] = find(Lab==cm);  % pixels of connected component 'cm'
        % bounding box of component 'cm'
        miny = min(r);    maxy = max(r);    minx = min(c);    maxx = max(c);
        Area = rectint([minx,miny,maxx-minx,maxy-miny], [L,T,R-L,B-T]);
        % remove whole blob if intersecting with window
        if Area>0,  fftest(r,c)=0;  end
    end
    imwrite(fftest,[pathWipe '\' camera sprintf('%05d',stp*tt-(stp-1)) '.png'],'png');
end
toc
