% These variables should be initialized here if not so in BuildGraph.m
stp=2; nFrames=2000; WnSz=64; camera='traffic01-'; res=16; ImSz=[240,320];
offset=0/stp; dT=floor(nFrames/stp); margin=WnSz/2; rt1=1; rt2=1;
% Second optimization to merge pathes
fps = 30;           % frames per second
Tb  = 2*floor(fps/stp);    % time elapsed before which it's not allowed to switch pathes
Ts  = 4*floor(fps/stp);    % number of frames after which it's irrelevant to keep 
                           % track of time elapsed for cuts (equivalent to 3 sec)
K   = size(costs,1);       % number of pathes
%====================   IMPORTANT !!!  =============================%
% initialized in 'scratch_for_results' from 'BuildGraph' and 'Visualize' runs
% costs=zeros(K,dT); jjpixes=zeros(K,stp*dT); iipixes=zeros(K,stp*dT);
% sspixes=zeros(K,stp*dT)
%===================================================================%
% coefficients of each path cost in the total cost function
coef= [1,1.05,1.1]; % we know we have only 3 paths for now
maxcst = 99;       % max value of cost
maxdst = WnSz;      % max distance after which frame switch is allowed without penalty
% original frames
pathFrm = 'C:\AVW-TrafficVid';
%pathFrm = 'C:\Documents and Settings\helalfy\My Documents\Research\Event Modeling\Nagia\ffjpgs';
%pathFrm = 'C:\MunichVid\Scene2Cam1';
% path to store cropped frames
pathOut = 'C:\Documents and Settings\helalfy\My Documents\Research\Thumbnailing Frames\AVW-2path';
%pathOut = 'C:\MunichTemp';
%pathOut = 'C:\MunichCrop\Scene2Cam1-temp';
% individual frames costs from cumulative ones
costFrm = zeros(size(costs));
for pp  = 1:K
    costFrm(pp,1)=costs(pp,1);
    for ccss=2:dT
        costFrm(pp,ccss)=costs(pp,ccss)-costs(pp,ccss-1);
    end
end

% create second graph for merged pathes (adjacency list)
fprintf('Building 2nd optimization graph...\n'); tic;
% each node is indexed by 'time index' through path, 'depth' at specific
% index (how long it has been there) and 'path index'. The number of
% neighbors per node is K (there can be a path switch to any path including
% staying at the same path)
GR2 = struct('next',zeros(dT-1,Ts,K,K), 'cost',zeros(dT-1,Ts,K,K));
% In the first 'Tb' frames, no path switch can occur
% A neighbor = 0 means NO neighbor (because indices start at 1)
for tt  = 1:Tb
    for pp  = 1:K
        GR2.next(tt,tt,pp,1) = pp;         % only one neighbor at same path
        GR2.cost(tt,tt,pp,1) = costFrm(pp,tt+1);  % cost of step in same path
    end
end
% remaining frames
for tt  = Tb+1:dT-1     % time
    % first 'Tb' steps in same path
    for dd  = 1:Tb      % depth (how long in same path)
        for pp  = 1:K   % pathes
            GR2.next(tt,dd,pp,1) = pp;  % remain in same path if depth<Tb
            GR2.cost(tt,dd,pp,1) = costFrm(pp,tt+1);    % cost of same path
        end
    end
    % if at same path for over 'Tb', you can change path
    for dd  = Tb+1:Ts           % depth (how long in same path)
        for pp  = 1:K           % pathes
            for nb  = 1:pp-1    % neighbors
                GR2.next(tt,dd,pp,nb) = nb;   % can switch to any path
                % compute cost function (3 parts)
                if costFrm(nb,tt+1)<maxcst
                    cfrm = costFrm(nb,tt+1)/coef(nb);   % cost of frame
                else, cfrm = costFrm(nb,tt+1)*3*coef(nb);  % penalize costy windows
                end
                cjmp = (Ts-dd)/(Ts-Tb)*costFrm(nb,tt+1);  % cost of jump
                windst = sqrt( (jjpixes(pp,stp*tt)-jjpixes(nb,stp*(tt+1)))^2 + ...
                    (iipixes(pp,stp*tt)-iipixes(nb,stp*(tt+1)))^2 ); % window distance
                if windst<maxdst        % cost of distance
                    cdst = (maxdst-windst)/maxdst*costFrm(nb,tt+1);
                else, cdst = 0;
                end
                GR2.cost(tt,dd,pp,nb) = cfrm+cjmp+cdst;
            end
            for nb  = pp+1:K    % neighbors
                GR2.next(tt,dd,pp,nb) = nb;   % can switch to any path
                % compute cost function (3 parts)
                if costFrm(nb,tt+1)<maxcst
                    cfrm = costFrm(nb,tt+1)/coef(nb);	% cost of frame
                else, cfrm = costFrm(nb,tt+1)*3*coef(nb);  % penalize costy windows
                end
                cjmp = (Ts-dd)/(Ts-Tb)*costFrm(nb,tt+1);  % cost of jump
                windst = sqrt( (jjpixes(pp,stp*tt)-jjpixes(nb,stp*(tt+1)))^2 + ...
                    (iipixes(pp,stp*tt)-iipixes(nb,stp*(tt+1)))^2 ); % window distance
                if windst<maxdst        % cost of distance
                    cdst = (maxdst-windst)/maxdst*costFrm(nb,tt+1);
                else, cdst = 0;
                end
                GR2.cost(tt,dd,pp,nb) = cfrm+cjmp+cdst;
            end
            % nb=pp means same path thus no jump costs
            GR2.next(tt,dd,pp,pp) = pp;
            GR2.cost(tt,dd,pp,pp) = costFrm(pp,tt+1); % cost of staying at same path
        end     % for-pp
    end     % for-dd
end     % for-tt
toc

% shortest path through second graph
fprintf('Computing final shortest path...\n');    tic
% vertex list with distances and  predecessors
V2  = struct('dist',Inf*ones(dT,Ts,K), 'pred',zeros(dT,Ts,K,2));
% stack to visit nodes in a depth first approach
Stk = zeros(K,4);    % distance and node number (triplet: tt,dd,pp)
% add dummy header node with cost equal to that of first nodes in each path.
% Initialize first set of nodes (1st in each path)
for pp  = 1:K
    Stk(pp,1)   = costFrm(pp,1);        % distance
    Stk(pp,2:4) = [1,1,pp];             % vertex triple index
    V2.dist(1,1,pp) = costFrm(pp,1);    % V2.pred = 0 (unchanged)
end

% Dijkstra's main loop
maxtt   = 1;    % keep track of max frame of visited nodes
maxchg  = true; % has 'maxtt' been updated in this iteration? (for display)
tt=1;
for dum = 1:(dT-1)*Ts*K   % for all graph vertices (at most) - faster than while-loop
    if maxchg
        fprintf('Processing frame %5d of %5d', maxtt,dT)
    end
    [minDist,minIdx] = min(Stk(:,1));    % find next closest node to source
    WW  = Stk(minIdx,2:4);          % vertex triple index
    tt=WW(1); dd=WW(2); pp=WW(3);   % spread in 3 variables
    Stk(minIdx,:) = [];         % entry will no longer be used
    if tt==dT, break, end       % reached last step in some path
    for nb  = 1:K               % for all edges from this node
        ppnx = GR2.next(tt,dd,pp,nb);   % neighbor's path
        if ppnx==0, break, end  % no neighbor
        ttnx=tt+1;              % next time index (always increases by 1)
        if ppnx==pp         % if no path change, depth increases
            if dd<Ts, ddnx=dd+1; else, ddnx=dd; end
        else, ddnx=1;       % else depth resets to 1
        end
        % distance check
        if V2.dist(tt,dd,pp) + GR2.cost(tt,dd,pp,nb) < V2.dist(ttnx,ddnx,ppnx)
            % update neighbor's dist
            V2.dist(ttnx,ddnx,ppnx)   = V2.dist(tt,dd,pp) + GR2.cost(tt,dd,pp,nb);
            V2.pred(ttnx,ddnx,ppnx,:) = [dd,pp];    % implicitly at 'tt'
            Stk = [V2.dist(ttnx,ddnx,ppnx), ttnx,ddnx,ppnx ; Stk];   % push on stack
        end
    end     % for neighbors
    if tt>maxtt, maxtt=tt; maxchg=true; % max depth in graph
    else maxchg=false; end      % don't change display
    if maxchg
        fprintf('\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b')
    end
end     % end main Dijstra's loop
fprintf('\n');

% Build merged path: we have final node's path 'pp'
merged  = zeros(1,dT);  % merged path with exactly 'dT' steps
merged(dT)  = pp;       % last step initialized
for tt  = dT:-1:2       % build path backwards
    pprev  = V2.pred(tt,dd,pp,2);    % predecessor node's path
    if pprev==0
        Error('Incomplete path. Broken link')
        return
    end
    merged(tt-1)  = pprev;      % prepend to merged path
    pp  = pprev;
    dd  = V2.pred(tt,dd,pp,1);  % predecessor node's depth
end
toc

% Visualization
fprintf('Visualizing shortest path...\n');    tic
if isempty(dir(pathOut))
	mkdir(pathOut);     % create output directory if not existent
end
% % get location of video-in-video
% found   = false;
% howmany = 0;    % how many optimal windows found not on boundary
% WnSz    = minWnSz + (VinVss-1)*(2*res);   % "original" square window sizes
% margin  = WnSz/2;       % frame margin where no windows allowed
% nH      = floor((ImSz(1)-2*margin)/res)+1;     % number of rows of windows
% nW      = floor((ImSz(2)-2*margin)/res)+1;     % number of cols of windows
% while ~found
%     [minE,ww] = min(VinV);
%     ii  = floor((ww-1)/nW)+1;   jj  = mod((ww-1),nW)+1;
%     if ii~=1 && ii~=nH && jj~=1 && jj~=nW   % not on boundary
%         howmany = howmany + 1;
%         VinV(ww)= inf;  % do not search again [for debug]
%     else        % lowest energy window that is on boundary
%         found   = true;
%     end
% end
% % center coordinates of video-in-video window
% VinVii	= (ii-1)*res+margin+1;    VinVjj	= (jj-1)*res+margin+1;
VinVii=0;   VinVjj=0;   % not used outside video-in-video mode

cols    = [{'y'}, {'b'}, {'g'}, {'r'}, {'m'}];  % color list
L = zeros(K,1); R = zeros(K,1); T = zeros(K,1); B = zeros(K,1);
for tt  = 1:stp*dT
    fprintf('Processing frame %5d of %5d', tt,stp*dT)
%    margin = sspixes(:,tt)/2;      % only for variable window size
    L=round(jjpixes(:,tt)-margin);      R=round(jjpixes(:,tt)+margin-1);
    T=round(iipixes(:,tt)-rt1*margin);	B=round(iipixes(:,tt)+rt2*margin-1);
    L  = max(L,1);    R  = min(R,ImSz(2));  % limit to frame
    T  = max(T,1);    B  = min(B,ImSz(1));  %    boundary
    frmin = imread([pathFrm '\' camera sprintf('%05d',tt+stp*offset) '.jpg']); % read frame
    sel = merged(ceil(tt/stp));     % selected path for this frame
    % mark cropped regions on original frame and cut out first one (selected) only
    rects=[L(sel), R(sel), T(sel), B(sel)];     % selected on top
    col2go=cols(sel);       % selected color
    for pp  = 1:K
        if pp ~= sel
            rects=[rects; L(pp),R(pp),T(pp),B(pp)]; col2go=[col2go,cols(pp)];
        end
    end
    frmout = drawRect(frmin, rects, col2go, VinVii,VinVjj,WnSz);
    imwrite(frmout,[pathOut '\' camera sprintf('%05d',tt+stp*offset) '.jpg'],'jpg');
    fprintf('\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b')
end
toc
