% create graph for shortest path through video

%clear all; pack;
WnSz    = 128;   % cropping window size 32x32 (can be [32 24] in the future)
WnArea  = WnSz*WnSz;        % cropping window area in pixels
margin  = WnSz/2;           % margin
% works on wiped frame differences
% pathIn  = 'C:\Documents and Settings\helalfy\My Documents\Research\Thumbnailing Frames\wipedFrameDiff';
pathIn  = 'C:\MunichWiped';
camera  = 'Cam3-'; frmt = 'png';    % camera number and file format
fftest  = imread([pathIn '\' camera sprintf('%05d',1) '.' frmt]);   % read test frame
ImSz    = size(fftest);     % frame size
res     = 8;        % resolution: pixels moved by cropping rectangle per step
nghb    = 9;        % number of permissible neighbors (self + eight neighboring)
dT      = 1000;     % number of frames allowed per graph
stp     = 2;        % step between frames to be computed
offset  = 0;        % frame where to start processing; normalized by 'stp'
% number of possible cropping windows per frame
nH      = floor((ImSz(1)-2*margin)/res);    % number of rows of windows
nW      = floor((ImSz(2)-2*margin)/res);    % number of cols of windows
nWin    = nH*nW;
% files needed for visualization
% original frames
%pathFrm = 'C:\Documents and Settings\helalfy\My Documents\Research\Event Modeling\Nagia\ffjpgs';
pathFrm = 'C:\MunichVid\Scene2Cam3';
% path to store marked frames
%pathOut = 'C:\Documents and Settings\helalfy\My Documents\Research\Thumbnailing Frames\temp';
%pathOut = 'C:\MunichTemp';

% create system graph (adjacency list)
fprintf('Building graph...\n'); tic;
GR      = struct('next',zeros(nWin,nghb,dT), 'cost',zeros(nWin,nghb,dT));
% in Help, it says that the above saves significant memory over this:
% GR      = repmat(struct('next',0, 'cost',0), [nWin,nghb,dT]);
for tt  = 1:dT-1      % for each frame in video
    fprintf('Processing frame %5d of %5d', tt,dT)
%     thisfr  = imread([pathIn '\' camera sprintf('%05d',stp*tt-(stp-1)) '.' frmt]);  % current diff image
    nextfr  = imread([pathIn '\' camera sprintf('%05d',stp*(tt+offset)+1) '.' frmt]); % next diff image
    % integral images
%     thisfr  = cumsum(cumsum(thisfr),2);
    nextfr  = cumsum(cumsum(nextfr),2);
    for ww  = 1:nWin        % for each window in this frame
        % get neighbors' list for input window in (i,j)-coordinates and
        % window number formats [variable size list 4, 6 or 9]
        [nblst]=neighb(ww, ImSz,margin,res);
        for nn  = 1:size(nblst,1)   % for all possible neighbors of window 'ww'
            ii = nblst(nn,2)*res+margin;   jj = nblst(nn,3)*res+margin; % center pixel
            % energy in next window computed as in Viola & Jones
            nexten  = nextfr(ii+margin,jj+margin) + nextfr(ii-margin,jj-margin) ...
                - nextfr(ii-margin,jj+margin) - nextfr(ii+margin,jj-margin);
            cost    = round(100*(1 - nexten/WnArea));   % my cost function (empty percentage)
            % add an edge to the graph. if (GR.next==0) edge doesn't exist
            GR.next(ww,nn,tt)=nblst(nn,1);  GR.cost(ww,nn,tt)=cost;
        end     % for-nn
    end     % for-ww
    fprintf('\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b')
end     % for-tt
GR.next(:,:,dT) = Inf*ones(nWin,nghb);  % last frame vertices connected to dummy sentinel vertex
toc;
% fprintf('Saving graph...\n');       tic
% save('Graph_1-15.mat', 'GR'); toc

% shortest path through graph
fprintf('Computing shortest path...\n');    tic
% vertex list with distances,  predecessors, and last-updated fields
V   = struct('dist',Inf*ones(nWin,dT), 'pred',zeros(nWin,dT), 'updt',zeros(nWin,dT));
% stack to visit nodes in a depth first approach
Stk = zeros(nWin,2);    % distance and global window number per vertex
% add dummy header node with cost=0; compute distances of first frame nodes
firstfr  = imread([pathIn '\' camera sprintf('%05d',stp*(0+offset)+1) '.' frmt]);  % first difference frame
firstfr  = cumsum(cumsum(firstfr),2);   % integral image
% first layer of nodes connected to dummy node
for WW  = 1:nWin
    % compute center pixel of this window
    jj  = mod((WW-1),nW)+1;    % column of window center
    ii  = floor((WW-1)/nW)+1;  % row of window center
    ii  = ii*res+margin;    jj  = jj*res+margin;    % actual center's pixel
    % energy in current window computed as in Viola & Jones
    thisen  = firstfr(ii+margin,jj+margin) + firstfr(ii-margin,jj-margin) ...
        - firstfr(ii-margin,jj+margin) - firstfr(ii+margin,jj-margin);
    Stk(WW,1)=round(100*(1 - thisen/WnArea));   Stk(WW,2)=WW;   % distance and window number
    V.dist(WW,1) = Stk(WW,1);   % all V.pred=0 unchanged
end

% Dijkstra's main loop
maxtt   = 0;    % keep track of max depth of visited nodes
maxchg  = true; % has 'maxtt' been updated in this iteration? (for display)
maxstk  = 0;    % max stack depth
tt      = 1;
for dum = 1 : nWin*dT   % for all graph vertices (at most) - faster than while-loop
    if maxchg
        fprintf('Processing frame %5d of %5d', maxtt,dT)
    end
    [minDist,minIdx]    = min(Stk(:,1));    % find next closest node to source
    WW  = Stk(minIdx,2);        % global window number (over all frames)
    Stk(minIdx,:) = [];         % entry will no longer be used
    tt  = floor((WW-1)/nWin)+1; % time frame of vertex
    ww  = mod(WW-1,nWin)+1;     % window within that frame
    if tt==dT, break, end       % reached last frame
    for nn  = 1:nghb            % for all edges from this node
        wwnb=GR.next(ww,nn,tt); ttnb=tt+1;  % neighbor's window and time frame
        if wwnb==0, break, end              % no neighbor; boundary window
        % neighbor is implicitly understood to be in frame tt+1
        if V.dist(ww,tt) + GR.cost(ww,nn,tt) < V.dist(wwnb,ttnb)   % distance check
            V.dist(wwnb,ttnb) = V.dist(ww,tt) + GR.cost(ww,nn,tt); % update neighbor's dist
            V.pred(wwnb,ttnb) = ww;    % 'ww' implicitly understood to be at frame 'tt'
            Stk = [V.dist(wwnb,ttnb), (ttnb-1)*nWin+wwnb ; Stk];   % push on stack
        end
    end     % for neighbors
    if ttnb>maxtt, maxtt=ttnb; end  % max depth in graph
    V.updt(ww,tt)   = maxtt;        % if zero then node hasn't been visited
    if size(Stk,1)>maxstk, maxstk=size(Stk,1); end
    if maxchg
        fprintf('\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b')
    end
end     % for graph nodes
fprintf('\n');

% Build path: we have final node's (ww,tt)
path=zeros(1,dT);   % path with exactly 'dT' steps
cost=zeros(1,dT);   % cumulative cost of path
path(dT)= ww;       % last step initialized
for tt  = dT:-1:2   % build path backwards
    pp  = V.pred(ww,tt);    % predecessor node
    if pp==0
        Error('Incomplete path. Broken link')
        return
    end
    path(tt-1)  = pp;       % prepend to path
    cost(tt)    = V.dist(ww,tt);
    ww  = pp;
end
cost(1) = V.dist(ww,1);
% Smooth path
jjpath=mod((path-1),nW)+1; iipath=floor((path-1)/nW)+1; % recover components
% smooth each component separately. 'floor' smoothes better than 'round'
% moving average smoothing with neighborhood size=9 to smooth jiggles
% Using 'floor' here in addition to the round in lines 128-129 helps better
% remove the jiggles
jjsmooth=floor(smooth(jjpath,9)); iismooth=floor(smooth(iipath,9));
% sampling indices, includes first and last
smpl=[1:5:dT]; if smpl(end)<dT, smpl=[smpl,dT];  end
% Hermite cubic interpolation of path over actual frames' pixels
jjpix = round(interp1( smpl,jjsmooth(smpl)*res+margin, [1:1/stp:dT], 'pchip' ));
iipix = round(interp1( smpl,iismooth(smpl)*res+margin, [1:1/stp:dT], 'pchip' ));
% last step actual frames (non interpolated)
lastjjpix=jjpix(end);   lastiipix=iipix(end);
for ss=1:stp-1
    jjpix=[jjpix,lastjjpix]; iipix=[iipix,lastiipix];
end
 %pathsmooth=(iismooth-1)*nW+jjsmooth;    % recombine in window numbers
toc;    % time to compute shortest path

% Pass II: Wipe enclosed "objects" from source frame differences
fprintf('Wiping frames, Pass III...\n');    tic
%pathWipe='C:\Documents and Settings\helalfy\My Documents\Research\Thumbnailing Frames\wipedFrameDiff';
pathWipe='C:\MunichWiped';
if isempty(dir(pathWipe))
	mkdir(pathWipe);     % create output directory if not existent
end
for tt  = 1:dT
    % border of cropping window; avoid that they round off out of window
    L=jjpix(stp*tt-(stp-1))-margin;     R=jjpix(stp*tt-(stp-1))+margin;
    T=iipix(stp*tt-(stp-1))-margin;     B=iipix(stp*tt-(stp-1))+margin;
    L=max(L,1);     R=min(R,ImSz(2));   T=max(T,1);     B=min(B,ImSz(1));
    % read one image difference frame
    fftest = imread([pathIn '\' camera sprintf('%05d',stp*tt-(stp-1)) '.png']);
    [Lab,ncmpn] = bwlabel(fftest,8);  % connected components
    for cm  = 1:ncmpn
        [r,c] = find(Lab==cm);  % pixels of connected component 'cm'
        % bounding box of component 'cm'
        miny = min(r);    maxy = max(r);    minx = min(c);    maxx = max(c);
        Area = rectint([minx,miny,maxx-minx,maxy-miny], [L,T,R-L,B-T]);
        % remove whole blob if intersecting with window
        if Area>0,  fftest(r,c)=0;  end
    end
    imwrite(fftest,[pathWipe '\' camera sprintf('%05d',stp*tt-(stp-1)) '.png'],'png');
end
toc
