function [imout] = drawRect(imin, RECT, col, I,J,ss)
% Draws rectangles low level in the RGB image imin. Zoom-in the first one
% of them and blow it out. Return result in 'imout'
% 'imin': original image (frame)
% 'RECT': array of rectangles (each rectangle in a row of xL,xR,yT,yB)
% 'col': cell array of colors
% 'I','J': coordinates of video-in-video window center
% 'ss': cropping window size (mainly needed for variable size cropping

%imrct=imin;
imrct=imresize(imin, [128,192], 'bicubic');
for ii  = size(RECT,1):-1:1
    switch col{ii}      % low level color components
        case 'y', rgbcol=[255,255,0];
        case 'm', rgbcol=[255,0,255];
        case 'c', rgbcol=[0,255,255];
        case 'gray', rgbcol=[127,127,127];
        case 'r', rgbcol=[255,0,0];
        case 'g', rgbcol=[0,255,0];
        case 'b', rgbcol=[0,0,255];
        case 'w', rgbcol=[255,255,255];
        case 'k', rgbcol=[0,0,0];
    end
    % original image with colored rectangle
    xL=floor(RECT(ii,1)*3/5); xR=floor(RECT(ii,2)*3/5); 
    yT=floor(RECT(ii,3)/1.875); yB=floor(RECT(ii,4)/1.875);
    imrct(yT,xL:xR,1)=rgbcol(1); imrct(yT,xL:xR,2)=rgbcol(2); imrct(yT,xL:xR,3)=rgbcol(3); % top line
    imrct(yB,xL:xR,1)=rgbcol(1); imrct(yB,xL:xR,2)=rgbcol(2); imrct(yB,xL:xR,3)=rgbcol(3); % bottom line
    imrct(yT:yB,xL,1)=rgbcol(1); imrct(yT:yB,xL,2)=rgbcol(2); imrct(yT:yB,xL,3)=rgbcol(3); % left line
    imrct(yT:yB,xR,1)=rgbcol(1); imrct(yT:yB,xR,2)=rgbcol(2); imrct(yT:yB,xR,3)=rgbcol(3); % right line
end

xL=round(2.25*RECT(1,1)); xR=round(2.25*RECT(1,2)); yT=2*RECT(1,3); yB=2*RECT(1,4);     % zoom-in 1st window
imcrp = imin(yT:yB,xL:xR,:);    % cropped image
% resize square cropped region to 1/3 original video size
%-> imcrp = imresize(imcrp, [round(size(imin,1)/3),round(size(imin,1)/3)], 'bicubic');
imcrp = imresize(imcrp, [2*ss,2*ss], 'bicubic');
% resize original video to 1/3 (chosen based on Munich video only)
%-> imrct = imresize(imrct, [round(size(imin,1)/3),round(size(imin,2)/3)], 'bicubic');

% % this one resizes original video to cropped square keeping its aspect ratio
% imrct = imresize(imrct, [size(imcrp,1), ...
%     round(size(imin,2)/size(imin,1)*size(imcrp,1))], 'bicubic');

% paste original video and cropped one side by side
% blkline=uint8(zeros(size(imin,1),4));        % black line, thickness = 4 pixels
% whtline=uint8(255*ones(size(imin,1),4));     % white line, thickness = 4 pixels
blkline=uint8(zeros(size(imcrp,1),3));        % black line, thickness = 3 pixels
whtline=uint8(255*ones(size(imcrp,1),2));     % white line, thickness = 2 pixels
sep=[blkline, whtline, blkline];    % separator, thickness = 8 pixels

% combined image; color by color
imout(:,:,1)=[imrct(:,:,1),sep,imcrp(:,:,1)];
imout(:,:,2)=[imrct(:,:,2),sep,imcrp(:,:,2)];
imout(:,:,3)=[imrct(:,:,3),sep,imcrp(:,:,3)];

% % paste cropped part inside original frame centered at (I,J)
% ss2=floor(ss/2);
% imrct(I-ss2:I+ss2-1, J-ss2:J+ss2-1, :) = imcrp;
% imout=imrct; 