function [nblst]=neighb(iiw,ssw, nH,nW, nSz)
% [nblst] = neighb(iiw, ImSz,margin,res)
%   returns neighbors' list for input window
%   input: iiw: window index
%          ImSz, margin, res: image size, margin and resolution
%   output: nblst: neighbors list - indices and (i,j) coordinates

jj  = mod((iiw-1),nW)+1;    % column of window center
ii  = floor((iiw-1)/nW)+1;  % row of window center
nblst=[iiw,ii,jj, ssw];     % neighbor list; start with self
% create neighbor list  (with min number of comparisons)
if jj>1         % far from left margin
    inb=ii; jnb=jj-1; wnb=(inb-1)*nW+jnb;   % left neighbor
    nblst=[nblst; wnb,inb,jnb, ssw];
    if ii>1     % and far from top margin
        inb=ii-1; jnb=jj-1; wnb=(inb-1)*nW+jnb; % top-left neighbor
        nblst=[nblst; wnb,inb,jnb, ssw];
    end
    if ii<nH    % and far from bottom margin
        inb=ii+1; jnb=jj-1; wnb=(inb-1)*nW+jnb; % bottom-left neighbor
        nblst=[nblst; wnb,inb,jnb, ssw];
    end
end
if jj<nW        % far from right margin
    inb=ii; jnb=jj+1; wnb=(inb-1)*nW+jnb;   % right neighbor
    nblst=[nblst; wnb,inb,jnb, ssw];
    if ii>1     % and far from top margin
        inb=ii-1; jnb=jj+1; wnb=(inb-1)*nW+jnb; % top-right neighbor
        nblst=[nblst; wnb,inb,jnb, ssw];
    end
    if ii<nH    % and far from bottom margin
        inb=ii+1; jnb=jj+1; wnb=(inb-1)*nW+jnb; % bottom-right neighbor
        nblst=[nblst; wnb,inb,jnb, ssw];
    end
end
if ii>1         % far from top margin    
    inb=ii-1; jnb=jj; wnb=(inb-1)*nW+jnb;   % top neighbor
    nblst=[nblst; wnb,inb,jnb, ssw];
end
if ii<nH        % far from bottom margin
    inb=ii+1; jnb=jj; wnb=(inb-1)*nW+jnb;   % bottom neighbor
    nblst=[nblst; wnb,inb,jnb, ssw];
end

if ssw>1        % can reduce size without restrictions
    % compute window indices in new scale coordinates
    ssnb=ssw-1;     jnb=jj+1;   inb=ii+1;
    nWnb=nW+2;      wnb=(inb-1)*nWnb+jnb;
    nblst=[nblst; wnb,inb,jnb, ssnb];
end
if ssw<nSz      % increase size if not on margin
    if ii>1 && ii<nH && jj>1 && jj<nW
        ssnb=ssw+1; jnb=jj-1;   inb=ii-1;
        nWnb=nW-2;  wnb=(inb-1)*nWnb+jnb;
        nblst=[nblst; wnb,inb,jnb, ssnb];
    end
end